@@ -4,15 +4,14 @@ import json
 import logging
 import socket
 import tempfile
+from six import iteritems
+from localstack.config import DEFAULT_REGION
+from localstack.utils.aws import aws_stack
 from localstack.utils.common import (short_uid, parallelize, is_port_open, new_tmp_file,
-    rm_rf, unzip, download, clean_cache, mktime, load_file, mkdir, run, md5)
+    to_str, rm_rf, unzip, download, clean_cache, mktime, load_file, mkdir, run, md5)
 from localstack.utils.aws.aws_models import (ElasticSearch, S3Notification,
     EventSource, DynamoDB, DynamoDBStream, FirehoseStream, S3Bucket, SqsQueue,
     KinesisShard, KinesisStream, LambdaFunction)
-from localstack.utils.aws import aws_stack
-from localstack.utils.common import to_str
-from localstack.constants import DEFAULT_REGION
-from six import iteritems
 
 # TODO: CLI commands in this file need to be replaced with SDK calls!
 
@@ -97,11 +96,11 @@ def aws_cmd(service, env):
     if aws_stack.is_local_env(env):
         endpoint_url = aws_stack.get_local_service_url(service)
     if endpoint_url:
+        if not is_port_open(endpoint_url):
+            raise socket.error()
         if endpoint_url.startswith('https://'):
             cmd += ' --no-verify-ssl'
         cmd = '%s --endpoint-url="%s"' % (cmd, endpoint_url)
-        if not is_port_open(endpoint_url):
-            raise socket.error()
     cmd = '%s %s' % (cmd, service)
     return cmd
 
@@ -150,8 +149,7 @@ def get_sqs_queues(filter='.*', pool={}, env=None):
         queues = json.loads(out)['QueueUrls']
         for q in queues:
             name = q.split('/')[-1]
-            account = q.split('/')[-2]
-            arn = 'arn:aws:sqs:%s:%s:%s' % (DEFAULT_REGION, account, name)
+            arn = aws_stack.sqs_queue_arn(name)
             if re.match(filter, name):
                 queue = SqsQueue(arn)
                 result.append(queue)
