@@ -10,22 +10,23 @@ import boto3
 import subprocess
 from requests.models import Response
 from localstack import constants, config
-from localstack.config import USE_SSL
 from localstack.constants import (
-    ENV_DEV, DEFAULT_REGION, LOCALSTACK_VENV_FOLDER, ENV_INTERNAL_TEST_RUN,
-    DEFAULT_PORT_APIGATEWAY_BACKEND, DEFAULT_PORT_SNS_BACKEND, DEFAULT_PORT_IAM_BACKEND)
+    ENV_DEV, LOCALSTACK_VENV_FOLDER, ENV_INTERNAL_TEST_RUN, LOCALSTACK_INFRA_PROCESS,
+    DEFAULT_PORT_APIGATEWAY_BACKEND, DEFAULT_PORT_SNS_BACKEND, DEFAULT_PORT_EVENTS_BACKEND,
+    DEFAULT_SERVICE_PORTS)
 from localstack.utils import common, persistence
-from localstack.utils.common import (TMP_THREADS, run, get_free_tcp_port,
-    FuncThread, ShellCommandThread, get_service_protocol, in_docker)
+from localstack.utils.common import (TMP_THREADS, run, get_free_tcp_port, is_linux,
+    FuncThread, ShellCommandThread, get_service_protocol, in_docker, is_port_open)
 from localstack.utils.server import multiserver
-from localstack.utils.bootstrap import setup_logging, is_debug, canonicalize_api_names
+from localstack.utils.bootstrap import (
+    setup_logging, is_debug, canonicalize_api_names, load_plugins, in_ci)
 from localstack.utils.analytics import event_publisher
-from localstack.utils.bootstrap import load_plugins
 from localstack.services import generic_proxy, install
 from localstack.services.es import es_api
+from localstack.services.plugins import SERVICE_PLUGINS, record_service_health, check_infra
 from localstack.services.firehose import firehose_api
 from localstack.services.awslambda import lambda_api
-from localstack.services.generic_proxy import GenericProxy, GenericProxyHandler
+from localstack.services.generic_proxy import GenericProxy, GenericProxyHandler, ProxyListener
 from localstack.services.dynamodbstreams import dynamodbstreams_api
 
 # flag to indicate whether signal handlers have been set up already
@@ -35,44 +36,7 @@ SIGNAL_HANDLERS_SETUP = False
 DEFAULT_BACKEND_HOST = '127.0.0.1'
 
 # set up logger
-LOG = logging.getLogger(os.path.basename(__file__))
-
-# map of service plugins, mapping from service name to plugin details
-SERVICE_PLUGINS = {}
-
-
-# -----------------
-# PLUGIN UTILITIES
-# -----------------
-
-
-class Plugin(object):
-
-    def __init__(self, name, start, check=None, listener=None):
-        self.plugin_name = name
-        self.start_function = start
-        self.listener = listener
-        self.check_function = check
-
-    def start(self, asynchronous):
-        kwargs = {
-            'asynchronous': asynchronous
-        }
-        if self.listener:
-            kwargs['update_listener'] = self.listener
-        return self.start_function(**kwargs)
-
-    def check(self, expect_shutdown=False, print_error=False):
-        if not self.check_function:
-            return
-        return self.check_function(expect_shutdown=expect_shutdown, print_error=print_error)
-
-    def name(self):
-        return self.plugin_name
-
-
-def register_plugin(plugin):
-    SERVICE_PLUGINS[plugin.name()] = plugin
+LOG = logging.getLogger(__name__)
 
 
 # -----------------------
@@ -80,7 +44,13 @@ def register_plugin(plugin):
 # -----------------------
 
 
-class ConfigUpdateProxyListener(object):
+def update_config_variable(variable, new_value):
+    if new_value is not None:
+        LOG.info('Updating value of config variable "%s": %s' % (variable, new_value))
+        setattr(config, variable, new_value)
+
+
+class ConfigUpdateProxyListener(ProxyListener):
     """ Default proxy listener that intercepts requests to retrieve or update config variables. """
 
     def forward_request(self, method, path, data, headers):
@@ -95,9 +65,7 @@ class ConfigUpdateProxyListener(object):
             response.status_code = 400
             return response
         new_value = data.get('value')
-        if new_value is not None:
-            LOG.info('Updating value of config variable "%s": %s' % (variable, new_value))
-            setattr(config, variable, new_value)
+        update_config_variable(variable, new_value)
         value = getattr(config, variable, None)
         result = {'variable': variable, 'value': value}
         response._content = json.dumps(result)
@@ -129,14 +97,10 @@ def start_cloudwatch(port=None, asynchronous=False):
     return start_moto_server('cloudwatch', port, name='CloudWatch', asynchronous=asynchronous)
 
 
-def start_cloudwatch_logs(port=None, asynchronous=False):
-    port = port or config.PORT_LOGS
-    return start_moto_server('logs', port, name='CloudWatch Logs', asynchronous=asynchronous)
-
-
-def start_events(port=None, asynchronous=False):
+def start_events(port=None, asynchronous=False, update_listener=None):
     port = port or config.PORT_EVENTS
-    return start_moto_server('events', port, name='CloudWatch Events', asynchronous=asynchronous)
+    return start_moto_server('events', port, name='CloudWatch Events', asynchronous=asynchronous,
+        backend_port=DEFAULT_PORT_EVENTS_BACKEND, update_listener=update_listener)
 
 
 def start_sts(port=None, asynchronous=False):
@@ -144,12 +108,6 @@ def start_sts(port=None, asynchronous=False):
     return start_moto_server('sts', port, name='STS', asynchronous=asynchronous)
 
 
-def start_iam(port=None, asynchronous=False, update_listener=None):
-    port = port or config.PORT_IAM
-    return start_moto_server('iam', port, name='IAM', asynchronous=asynchronous,
-        backend_port=DEFAULT_PORT_IAM_BACKEND, update_listener=update_listener)
-
-
 def start_redshift(port=None, asynchronous=False):
     port = port or config.PORT_REDSHIFT
     return start_moto_server('redshift', port, name='Redshift', asynchronous=asynchronous)
@@ -195,19 +153,74 @@ def start_secretsmanager(port=None, asynchronous=False):
     return start_moto_server('secretsmanager', port, name='Secrets Manager', asynchronous=asynchronous)
 
 
-def start_ec2(port=None, asynchronous=False):
-    port = port or config.PORT_EC2
-    return start_moto_server('ec2', port, name='EC2', asynchronous=asynchronous)
-
-
 # ---------------
 # HELPER METHODS
 # ---------------
 
+def patch_urllib3_connection_pool(**constructor_kwargs):
+    """
+    Override the default parameters of HTTPConnectionPool, e.g., set the pool size via maxsize=16
+    """
+    try:
+        from urllib3 import connectionpool, poolmanager
 
-def restore_persisted_data(apis):
-    for api in apis:
-        persistence.restore_persisted_data(api)
+        class MyHTTPSConnectionPool(connectionpool.HTTPSConnectionPool):
+            def __init__(self, *args, **kwargs):
+                kwargs.update(constructor_kwargs)
+                super(MyHTTPSConnectionPool, self).__init__(*args, **kwargs)
+        poolmanager.pool_classes_by_scheme['https'] = MyHTTPSConnectionPool
+
+        class MyHTTPConnectionPool(connectionpool.HTTPConnectionPool):
+            def __init__(self, *args, **kwargs):
+                kwargs.update(constructor_kwargs)
+                super(MyHTTPConnectionPool, self).__init__(*args, **kwargs)
+        poolmanager.pool_classes_by_scheme['http'] = MyHTTPConnectionPool
+    except Exception:
+        pass
+
+
+def set_service_status(data):
+    command = data.get('command')
+    service = data.get('service')
+    service_ports = config.parse_service_ports()
+    if command == 'start':
+        existing = service_ports.get(service)
+        port = DEFAULT_SERVICE_PORTS.get(service)
+        if existing:
+            status = get_service_status(service, port)
+            if status == 'running':
+                return
+        key_upper = service.upper().replace('-', '_')
+        port_variable = 'PORT_%s' % key_upper
+        service_list = os.environ.get('SERVICES', '').strip()
+        services = [e for e in re.split(r'[\s,]+', service_list) if e]
+        contained = [s for s in services if s.startswith(service)]
+        if not contained:
+            services.append(service)
+        update_config_variable(port_variable, port)
+        new_service_list = ','.join(services)
+        os.environ['SERVICES'] = new_service_list
+        config.populate_configs()
+        LOG.info('Starting service %s on port %s' % (service, port))
+        SERVICE_PLUGINS[service].start(asynchronous=True)
+    return {}
+
+
+def get_services_status():
+    result = {}
+    for service, port in config.parse_service_ports().items():
+        status = get_service_status(service, port)
+        result[service] = {
+            'port': port,
+            'status': status
+        }
+    return result
+
+
+def get_service_status(service, port=None):
+    port = port or config.parse_service_ports().get(service)
+    status = 'disabled' if (port or 0) <= 0 else 'running' if is_port_open(port) else 'stopped'
+    return status
 
 
 def register_signal_handlers():
@@ -224,17 +237,17 @@ def register_signal_handlers():
     SIGNAL_HANDLERS_SETUP = True
 
 
-def do_run(cmd, asynchronous, print_output=False, env_vars={}):
+def do_run(cmd, asynchronous, print_output=None, env_vars={}):
     sys.stdout.flush()
     if asynchronous:
-        if is_debug():
+        if is_debug() and print_output is None:
             print_output = True
         outfile = subprocess.PIPE if print_output else None
         t = ShellCommandThread(cmd, outfile=outfile, env_vars=env_vars)
         t.start()
         TMP_THREADS.append(t)
         return t
-    return run(cmd)
+    return run(cmd, env_vars=env_vars)
 
 
 def start_proxy_for_service(service_name, port, default_backend_port, update_listener, quiet=False, params={}):
@@ -244,9 +257,10 @@ def start_proxy_for_service(service_name, port, default_backend_port, update_lis
     return start_proxy(port, backend_url=backend_url, update_listener=update_listener, quiet=quiet, params=params)
 
 
-def start_proxy(port, backend_url, update_listener, quiet=False, params={}):
+def start_proxy(port, backend_url, update_listener=None, quiet=False, params={}, use_ssl=None):
+    use_ssl = config.USE_SSL if use_ssl is None else use_ssl
     proxy_thread = GenericProxy(port=port, forward_url=backend_url,
-        ssl=USE_SSL, update_listener=update_listener, quiet=quiet, params=params)
+        ssl=use_ssl, update_listener=update_listener, quiet=quiet, params=params)
     proxy_thread.start()
     TMP_THREADS.append(proxy_thread)
     return proxy_thread
@@ -256,7 +270,7 @@ def start_moto_server(key, port, name=None, backend_port=None, asynchronous=Fals
     if not name:
         name = key
     print('Starting mock %s (%s port %s)...' % (name, get_service_protocol(), port))
-    if USE_SSL and not backend_port:
+    if config.USE_SSL and not backend_port:
         backend_port = get_free_tcp_port()
     if backend_port:
         start_proxy_for_service(key, port, backend_port, update_listener)
@@ -316,34 +330,6 @@ def check_aws_credentials():
     assert credentials
 
 
-# -----------------------------
-# INFRASTRUCTURE HEALTH CHECKS
-# -----------------------------
-
-
-def check_infra(retries=10, expect_shutdown=False, apis=None, additional_checks=[]):
-    try:
-        print_error = retries <= 0
-
-        # loop through plugins and check service status
-        for name, plugin in SERVICE_PLUGINS.items():
-            if name in apis:
-                try:
-                    plugin.check(expect_shutdown=expect_shutdown, print_error=print_error)
-                except Exception as e:
-                    LOG.warning('Service "%s" not yet available, retrying...' % name)
-                    raise e
-
-        for additional in additional_checks:
-            additional(expect_shutdown=expect_shutdown)
-    except Exception as e:
-        if retries <= 0:
-            LOG.error('Error checking state of local environment (after some retries): %s' % traceback.format_exc())
-            raise e
-        time.sleep(3)
-        check_infra(retries - 1, expect_shutdown=expect_shutdown, apis=apis, additional_checks=additional_checks)
-
-
 # -------------
 # MAIN STARTUP
 # -------------
@@ -351,10 +337,23 @@ def check_infra(retries=10, expect_shutdown=False, apis=None, additional_checks=
 
 def start_infra(asynchronous=False, apis=None):
     try:
+        os.environ[LOCALSTACK_INFRA_PROCESS] = '1'
+
+        is_in_docker = in_docker()
+        # print a warning if we're not running in Docker but using Docker based LAMBDA_EXECUTOR
+        if not is_in_docker and 'docker' in config.LAMBDA_EXECUTOR and not is_linux():
+            print(('!WARNING! - Running outside of Docker with LAMBDA_EXECUTOR=%s can lead to '
+                   'problems on your OS. The environment variable $LOCALSTACK_HOSTNAME may not '
+                   'be properly set in your Lambdas.') % config.LAMBDA_EXECUTOR)
+
+        # apply patches
+        patch_urllib3_connection_pool(maxsize=128)
+
         # load plugins
         load_plugins()
 
-        event_publisher.fire_event(event_publisher.EVENT_START_INFRA, {'d': in_docker() and 1 or 0})
+        event_publisher.fire_event(event_publisher.EVENT_START_INFRA,
+            {'d': is_in_docker and 1 or 0, 'c': in_ci() and 1 or 0})
 
         # set up logging
         setup_logging()
@@ -362,7 +361,7 @@ def start_infra(asynchronous=False, apis=None):
         # prepare APIs
         apis = canonicalize_api_names(apis)
         # set environment
-        os.environ['AWS_REGION'] = DEFAULT_REGION
+        os.environ['AWS_REGION'] = config.DEFAULT_REGION
         os.environ['ENV'] = ENV_DEV
         # register signal handlers
         if not os.environ.get(ENV_INTERNAL_TEST_RUN):
@@ -376,12 +375,10 @@ def start_infra(asynchronous=False, apis=None):
         # start services
         thread = None
 
-        if 'elasticsearch' in apis or 'es' in apis:
-            sleep_time = max(sleep_time, 10)
-
         # loop through plugins and start each service
         for name, plugin in SERVICE_PLUGINS.items():
-            if name in apis:
+            if plugin.is_enabled(api_names=apis):
+                record_service_health(name, 'starting')
                 t1 = plugin.start(asynchronous=True)
                 thread = thread or t1
 
@@ -389,7 +386,7 @@ def start_infra(asynchronous=False, apis=None):
         # ensure that all infra components are up and running
         check_infra(apis=apis)
         # restore persisted data
-        restore_persisted_data(apis=apis)
+        persistence.restore_persisted_data(apis=apis)
         print('Ready.')
         sys.stdout.flush()
         if not asynchronous and thread:
