@@ -9,6 +9,7 @@ from six import iteritems
 from six import string_types
 from localstack.utils import common
 from localstack.utils.aws import aws_stack
+from localstack.services.s3 import s3_listener
 from localstack.utils.testutil import create_zip_file
 from localstack.services.awslambda.lambda_api import get_handler_file_from_name
 
@@ -36,12 +37,14 @@ def str_or_none(o):
     return o if o is None else json.dumps(o) if isinstance(o, (dict, list)) else str(o)
 
 
-def select_attributes(obj, attrs):
-    result = {}
-    for attr in attrs:
-        if obj.get(attr) is not None:
-            result[attr] = str_or_none(obj.get(attr))
-    return result
+def params_select_attributes(*attrs):
+    def do_select(params, **kwargs):
+        result = {}
+        for attr in attrs:
+            if params.get(attr) is not None:
+                result[attr] = str_or_none(params.get(attr))
+        return result
+    return do_select
 
 
 def get_bucket_location_config(**kwargs):
@@ -61,7 +64,7 @@ def rename_params(func, rename_map):
     return do_rename
 
 
-def params_list_to_dict(param_name, key_attr_name, value_attr_name):
+def params_list_to_dict(param_name, key_attr_name='Key', value_attr_name='Value'):
     def do_replace(params, **kwargs):
         result = {}
         for entry in params.get(param_name, []):
@@ -72,9 +75,28 @@ def params_list_to_dict(param_name, key_attr_name, value_attr_name):
     return do_replace
 
 
-def get_nested_stack_name(params, **kwargs):
+def params_dict_to_list(param_name, key_attr_name='Key', value_attr_name='Value', wrapper=None):
+    def do_replace(params, **kwargs):
+        result = []
+        for key, value in params.get(param_name, {}).items():
+            result.append({key_attr_name: key, value_attr_name: value})
+        if wrapper:
+            result = {wrapper: result}
+        return result
+    return do_replace
+
+
+def get_nested_stack_params(params, **kwargs):
     stack_name = kwargs.get('stack_name', 'stack')
-    return '%s-%s' % (stack_name, common.short_uid())
+    nested_stack_name = '%s-%s' % (stack_name, common.short_uid())
+    stack_params = params.get('Parameters', {})
+    stack_params = [{'ParameterKey': k, 'ParameterValue': v} for k, v in stack_params.items()]
+    result = {
+        'StackName': nested_stack_name,
+        'TemplateURL': params.get('TemplateURL'),
+        'Parameters': stack_params
+    }
+    return result
 
 
 def get_lambda_code_param(params, **kwargs):
@@ -100,10 +122,53 @@ def sns_subscription_params(params, **kwargs):
     return result
 
 
+def s3_bucket_notification_config(params, **kwargs):
+    notif_config = params.get('NotificationConfiguration')
+    if not notif_config:
+        return None
+
+    lambda_configs = []
+    queue_configs = []
+    topic_configs = []
+
+    attr_tuples = (
+        ('LambdaConfigurations', lambda_configs, 'LambdaFunctionArn', 'Function'),
+        ('QueueConfigurations', queue_configs, 'QueueArn', 'Queue'),
+        ('TopicConfigurations', topic_configs, 'TopicArn', 'Topic')
+    )
+
+    # prepare lambda/queue/topic notification configs
+    for attrs in attr_tuples:
+        for config in notif_config.get(attrs[0]) or []:
+            filter_rules = config.get('Filter', {}).get('S3Key', {}).get('Rules')
+            entry = {
+                attrs[2]: config[attrs[3]],
+                'Events': [config['Event']]
+            }
+            if filter_rules:
+                entry['Filter'] = {'Key': {'FilterRules': filter_rules}}
+            attrs[1].append(entry)
+
+    # construct final result
+    result = {
+        'Bucket': params.get('BucketName') or PLACEHOLDER_RESOURCE_NAME,
+        'NotificationConfiguration': {
+            'LambdaFunctionConfigurations': lambda_configs,
+            'QueueConfigurations': queue_configs,
+            'TopicConfigurations': topic_configs
+        }
+    }
+    return result
+
+
 def select_parameters(*param_names):
     return lambda params, **kwargs: dict([(k, v) for k, v in params.items() if k in param_names])
 
 
+def merge_parameters(func1, func2):
+    return lambda params, **kwargs: common.merge_dicts(func1(params, **kwargs), func2(params, **kwargs))
+
+
 def dump_json_params(param_func=None, *param_names):
     def replace(params, **kwargs):
         result = param_func(params, **kwargs) if param_func else params
@@ -130,14 +195,17 @@ def param_defaults(param_func, defaults):
 # maps resource types to functions and parameters for creation
 RESOURCE_TO_FUNCTION = {
     'S3::Bucket': {
-        'create': {
+        'create': [{
             'function': 'create_bucket',
             'parameters': {
                 'Bucket': ['BucketName', PLACEHOLDER_RESOURCE_NAME],
                 'ACL': lambda params, **kwargs: convert_acl_cf_to_s3(params.get('AccessControl', 'PublicRead')),
                 'CreateBucketConfiguration': lambda params, **kwargs: get_bucket_location_config()
             }
-        },
+        }, {
+            'function': 'put_bucket_notification_configuration',
+            'parameters': s3_bucket_notification_config
+        }],
         'delete': {
             'function': 'delete_bucket',
             'parameters': {
@@ -156,11 +224,9 @@ RESOURCE_TO_FUNCTION = {
             'function': 'create_queue',
             'parameters': {
                 'QueueName': ['QueueName', PLACEHOLDER_RESOURCE_NAME],
-                'Attributes': lambda params, **kwargs: select_attributes(params,
-                    ['DelaySeconds', 'MaximumMessageSize', 'MessageRetentionPeriod',
-                     'VisibilityTimeout', 'RedrivePolicy']
-                ),
-                'tags': params_list_to_dict('Tags', 'Key', 'Value')
+                'Attributes': params_select_attributes('DelaySeconds', 'MaximumMessageSize',
+                    'MessageRetentionPeriod', 'VisibilityTimeout', 'RedrivePolicy'),
+                'tags': params_list_to_dict('Tags')
             }
         },
         'delete': {
@@ -185,6 +251,13 @@ RESOURCE_TO_FUNCTION = {
             }
         }
     },
+    'SSM::Parameter': {
+        'create': {
+            'function': 'put_parameter',
+            'parameters': merge_parameters(params_dict_to_list('Tags', wrapper='Tags'), params_select_attributes(
+                'Name', 'Type', 'Value', 'Description', 'AllowedPattern', 'Policies', 'Tier'))
+        }
+    },
     'Logs::LogGroup': {
         # TODO implement
     },
@@ -391,19 +464,16 @@ RESOURCE_TO_FUNCTION = {
     'CloudFormation::Stack': {
         'create': {
             'function': 'create_stack',
-            'parameters': {
-                'StackName': get_nested_stack_name,
-                'TemplateURL': 'TemplateURL'
-            }
+            'parameters': get_nested_stack_params
         }
     }
 }
 
-
 # ----------------
 # UTILITY METHODS
 # ----------------
 
+
 def convert_acl_cf_to_s3(acl):
     """ Convert a CloudFormation ACL string (e.g., 'PublicRead') to an S3 ACL string (e.g., 'public-read') """
     return re.sub('(?<!^)(?=[A-Z])', '-', acl).lower()
@@ -469,7 +539,7 @@ def get_resource_name(resource):
 
     # try to extract name from attributes
     if res_type == 'S3::Bucket':
-        name = properties.get('BucketName')
+        name = s3_listener.normalize_bucket_name(properties.get('BucketName'))
     elif res_type == 'SQS::Queue':
         name = properties.get('QueueName')
     elif res_type == 'Cognito::UserPool':
@@ -518,6 +588,8 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
     resource_props = resource.get('Properties')
     try:
         if resource_type == 'Lambda::Function':
+            resource_props['FunctionName'] = (resource_props.get('FunctionName') or
+                '{}-lambda-{}'.format(stack_name[:45], common.short_uid()))
             resource_id = resource_props['FunctionName'] if resource else resource_id
             return aws_stack.connect_to_service('lambda').get_function(FunctionName=resource_id)
         elif resource_type == 'Lambda::Version':
@@ -547,6 +619,10 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             role_name = resource_props.get('RoleName') or resource_id
             role_name = resolve_refs_recursively(stack_name, role_name, resources)
             return aws_stack.connect_to_service('iam').get_role(RoleName=role_name)['Role']
+        elif resource_type == 'SSM::Parameter':
+            param_name = resource_props.get('Name') or resource_id
+            param_name = resolve_refs_recursively(stack_name, param_name, resources)
+            return aws_stack.connect_to_service('ssm').get_parameter(Name=param_name)['Parameter']
         elif resource_type == 'DynamoDB::Table':
             table_name = resource_props.get('TableName') or resource_id
             table_name = resolve_refs_recursively(stack_name, table_name, resources)
@@ -586,12 +662,12 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             if not api_id or not res_id:
                 return None
             res_obj = aws_stack.connect_to_service('apigateway').get_resource(restApiId=api_id, resourceId=res_id)
-            match = [v for (k, v) in res_obj['resourceMethods'].items()
+            match = [v for (k, v) in res_obj.get('resourceMethods', {}).items()
                      if resource_props['HttpMethod'] in (v.get('httpMethod'), k)]
-            int_props = resource_props.get('Integration')
-            if int_props:
+            int_props = resource_props.get('Integration') or {}
+            if int_props.get('Type') == 'AWS_PROXY':
                 match = [m for m in match if
-                    m.get('methodIntegration', {}).get('type') == int_props.get('Type') and
+                    m.get('methodIntegration', {}).get('type') == 'AWS_PROXY' and
                     m.get('methodIntegration', {}).get('httpMethod') == int_props.get('IntegrationHttpMethod')]
             return any(match) or None
         elif resource_type == 'ApiGateway::GatewayResponse':
@@ -617,7 +693,18 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
         elif resource_type == 'S3::Bucket':
             bucket_name = resource_props.get('BucketName') or resource_id
             bucket_name = resolve_refs_recursively(stack_name, bucket_name, resources)
-            return aws_stack.connect_to_service('s3').get_bucket_location(Bucket=bucket_name)
+            bucket_name = s3_listener.normalize_bucket_name(bucket_name)
+            s3_client = aws_stack.connect_to_service('s3')
+            response = s3_client.get_bucket_location(Bucket=bucket_name)
+            notifs = resource_props.get('NotificationConfiguration')
+            if not response or not notifs:
+                return response
+            configs = s3_client.get_bucket_notification_configuration(Bucket=bucket_name)
+            has_notifs = (configs.get('TopicConfigurations') or configs.get('QueueConfigurations') or
+                configs.get('LambdaFunctionConfigurations'))
+            if notifs and not has_notifs:
+                return None
+            return response
         elif resource_type == 'S3::BucketPolicy':
             bucket_name = resource_props.get('Bucket') or resource_id
             bucket_name = resolve_refs_recursively(stack_name, bucket_name, resources)
@@ -937,6 +1024,10 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
     # assign default values if empty
     params = common.merge_recursive(defaults, params)
 
+    # this is an indicator that we should skip this resource deployment, and return
+    if params is None:
+        return
+
     # convert refs and boolean strings
     for param_key, param_value in dict(params).items():
         if param_value is not None:
@@ -964,16 +1055,25 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
     # some resources have attached/nested resources which we need to create recursively now
     if resource_type == 'ApiGateway::Method':
         integration = resource_props.get('Integration')
+        apigateway = aws_stack.connect_to_service('apigateway')
         if integration:
             api_id = resolve_refs_recursively(stack_name, resource_props['RestApiId'], resources)
             res_id = resolve_refs_recursively(stack_name, resource_props['ResourceId'], resources)
-            uri = integration.get('Uri')
-            if uri:
-                uri = resolve_refs_recursively(stack_name, uri, resources)
-                aws_stack.connect_to_service('apigateway').put_integration(
-                    restApiId=api_id, resourceId=res_id,
-                    httpMethod=resource_props['HttpMethod'], type=integration['Type'],
-                    integrationHttpMethod=integration['IntegrationHttpMethod'], uri=uri)
+            kwargs = {}
+            if integration.get('Uri'):
+                uri = resolve_refs_recursively(stack_name, integration.get('Uri'), resources)
+                kwargs['uri'] = uri
+            if integration.get('IntegrationHttpMethod'):
+                kwargs['integrationHttpMethod'] = integration['IntegrationHttpMethod']
+            apigateway.put_integration(restApiId=api_id, resourceId=res_id,
+                httpMethod=resource_props['HttpMethod'], type=integration['Type'], **kwargs)
+        responses = resource_props.get('MethodResponses') or []
+        for response in responses:
+            api_id = resolve_refs_recursively(stack_name, resource_props['RestApiId'], resources)
+            res_id = resolve_refs_recursively(stack_name, resource_props['ResourceId'], resources)
+            apigateway.put_method_response(restApiId=api_id, resourceId=res_id,
+                httpMethod=resource_props['HttpMethod'], statusCode=response['StatusCode'],
+                responseParameters=response.get('ResponseParameters', {}))
     elif resource_type == 'SNS::Topic':
         subscriptions = resource_props.get('Subscription', [])
         for subscription in subscriptions:
